
#include "VideoView.h"
#include "qcam_driver.h"

#define bmsgVideoFrame 'vidf'

VideoView::VideoView(BRect r, ulong size, int bpp)
	: BView(r,"VideoView",B_FOLLOW_ALL,B_WILL_DRAW)
{
	m_qcamDevice = open("/dev/quickcam",O_RDONLY|O_BINARY);
	qcam qc;
	
	ioctl(m_qcamDevice,QC_GET,&qc);
	
	qc.brightness = 160;
	qc.contrast = 50;
	qc.whitebalance = 130;
	switch (size) {
		case QCAM_80x60:
			qc.factor = QC_FACTOR_41;
			m_numPix = 80*60;
			m_bitmap1 = new BBitmap(BRect(0,0,79,59),B_COLOR_8_BIT,FALSE);
			m_bitmap2 = new BBitmap(BRect(0,0,79,59),B_COLOR_8_BIT,FALSE);
			break;
		case QCAM_160x120:
			qc.factor = QC_FACTOR_21;
			m_numPix = 160*120;
			m_bitmap1 = new BBitmap(BRect(0,0,159,119),B_COLOR_8_BIT,FALSE);
			m_bitmap2 = new BBitmap(BRect(0,0,159,119),B_COLOR_8_BIT,FALSE);
			break;
		case QCAM_320x240:
			qc.factor = QC_FACTOR_11;
			m_numPix = 320*240;
			m_bitmap1 = new BBitmap(BRect(0,0,319,239),B_COLOR_8_BIT,FALSE);
			m_bitmap2 = new BBitmap(BRect(0,0,319,239),B_COLOR_8_BIT,FALSE);
			break;
	};

	m_bitmap = m_bitmap1;
	m_nextBitmap = m_bitmap2;
	switch (bpp) {
		case 4:
			qc.bpp = QC_4BIT_PIXEL;
			break;
		case 6:
			qc.bpp = QC_6BIT_PIXEL;
			break;
	};
	
	ioctl(m_qcamDevice,QC_SET,&qc);
};

VideoView::~VideoView()
{
	close(m_qcamDevice);
	delete m_bitmap;
	delete m_nextBitmap;
};

void VideoView::AttachedToWindow()
{
	inherited::AttachedToWindow();
	Window()->PostMessage(new BMessage(bmsgVideoFrame),this);
	Window()->ResizeTo(m_bitmap->Bounds().Width(),m_bitmap->Bounds().Height());
};

void VideoView::DoFrame()
{
	read(m_qcamDevice,m_nextBitmap->Bits(),m_numPix);
	BBitmap *bm = m_bitmap;
	m_bitmap = m_nextBitmap;
	m_nextBitmap = bm;
	DrawBitmapAsync(m_bitmap,m_bitmap->Bounds(),Bounds());
};

void VideoView::MessageReceived(BMessage *msg)
{
	switch (msg->what) {
		case bmsgVideoFrame:
			DoFrame();
			Window()->PostMessage(new BMessage(bmsgVideoFrame),this);
			break;
		default:
			inherited::MessageReceived(msg);
	};
};

void VideoView::Draw(BRect updateRect)
{
	DrawBitmap(m_bitmap,m_bitmap->Bounds(),Bounds());
};
